/* Fakiir
 *
 * (C) Copyright 2005 Denis Rojo <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <pthread.h>

#include <daemons.h>




//////// XMLRPC

void *xmlrpc_handler(void *data) {
  Srv_XmlRpc *xmlrpc = (Srv_XmlRpc*)data;
  while(!xmlrpc->quit)
    xmlrpc->srv->work(100.0); // wait for request 100 ms
  return NULL;
}

Srv_XmlRpc::Srv_XmlRpc(Fakiir *fak) {
  running = false;
  quit = false;

  srv = new XmlRpcServer();

  // DEBUG
  XmlRpc::setVerbosity(5);

  fakiir = fak;

  pthread_attr_init(&_attr); // if -1 is error
  pthread_attr_setdetachstate(&_attr,PTHREAD_CREATE_JOINABLE);
}

Srv_XmlRpc::~Srv_XmlRpc() {
  delete srv;
}

bool Srv_XmlRpc::start(int port) {
  if(running) return true;
 
  srv->bindAndListen(port);

  srv->enableIntrospection(true);

  pthread_create(&_thread, &_attr, &xmlrpc_handler, this);

  running = true;

  return running;
}

void Srv_XmlRpc::stop() {

  if(!running) return;
  quit = true;
  pthread_join(_thread,NULL);
  running = false;

}





//////// LIBLO

int lo_handler(const char *path, const char *types, lo_arg **argv,
		  int argc, void *data, void *user_data) {

  Object *o;
  Method *m;

  Argument *a;
  Argument_map::iterator a_iter;

  Fakiir *f = (Fakiir*)user_data;

  char *target;

  target = strtok((char*)path,"/");
  
  // find our container
  o = (*f)[target];
  if(!o) {
    fprintf(stderr,"container %s is not available\n",target);
    return 0;
  }
  
  target = strtok(NULL,"/");

  // find our methods
  m = o->methods[target];

  if(!m) {
    fprintf(stderr,"method %s is not available\n",target);
    return 0;
  }

  // cycle thru the arguments
  int c = 1;
  for(a_iter = m->args.begin(); a_iter != m->args.end(); ++a_iter, c++) {
    a = (*a_iter);

    target = strtok(NULL,"/");
 
    if(!target) {
      fprintf(stderr,"missing arg %u in call to /%s/%s: expecting %s type\n",
	      c, o->name.c_str(), m->name.c_str(),
	      (a->type==INT_VALUE)?"integer":
	      (a->type==FLOAT_VALUE)?"float":
	      (a->type==STRING_VALUE)?"string":"unknown");
      return(0);
    }

    if( !a->set(target) ) {
      fprintf(stderr,"malformed arg %u in call to /%s/%s: %s is not a %s\n",
	      c, o->name.c_str(), m->name.c_str(), target,
	      (a->type==INT_VALUE)?"integer":
	      (a->type==FLOAT_VALUE)?"float":
	      (a->type==STRING_VALUE)?"string":"??");
      return(0);
    }
       
  }
 
  m->exec();
  
  return 1;
}

void lo_error(int num, const char *m, const char *path) {
  fprintf(stderr,"liblo error %u: %s %s\n",num,path,m);
}

Srv_Osc::Srv_Osc(Fakiir *fak) {
  running = false;
  srv = NULL;

  fakiir = fak;
}

Srv_Osc::~Srv_Osc() {
  stop();
}

bool Srv_Osc::start(int port) {
  if(running) return true;

  char tmp[64];
  snprintf(tmp,63,"%u",port);  

  srv = lo_server_thread_new(tmp,lo_error);
  
  lo_server_thread_add_method(srv,NULL,NULL,lo_handler,fakiir);

  lo_server_thread_start(srv);
  
  running = true;
  return running;
}

void Srv_Osc::stop() {
  if(!running) return;
  lo_server_thread_stop(srv);
  running = false;
}
			
