/* Fakiir
 *
 * (C) Copyright 2005 Denis Rojo <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * "$Id: $"
 *
 */
  
#include <iostream>
#include <cstdio>
#include <stdlib.h>

#include <fakiir.h>
#include <daemons.h>
#include <transfers.h>

Fakiir::Fakiir() {

#ifndef FAKIIR_GENESIS
  xmlrpc = new Srv_XmlRpc(this);
  osc = new Srv_Osc(this);
#endif

}

Fakiir::~Fakiir() {

#ifndef FAKIIR_GENESIS
  if(xmlrpc) delete xmlrpc;
  if(osc) delete osc;
#endif

}

bool Fakiir::init(string name, void *real) {
  Object *o = NULL;
  o = search(name);
  if(!o) {
    fprintf(stderr,"error at initialization with object %s: not found\n",name.c_str());
    return false;
  }
  o->real = real;
  return true;
}

Object *Fakiir::search(string name) {
  /// @todo: search all tree descending childs
  return (*tree)[name.c_str()];
}

#ifndef FAKIIR_GENESIS
void Fakiir::launch_daemons() {
  xmlrpc->start(7770);
  osc->start(7770);
}
#endif

Object::Object(Fakiir *fak, string n) {
	parent = NULL;
	fakiir = fak;
	name = n;
}

Object::~Object() {
  // @todo: destruction of all tree below?
}

/////


Method::Method(Object *o, string n) {
  parent = o;
  name = n;

#ifndef FAKIIR_GENESIS  
  // initialize transfers
  xmlrpc = new TransXmlRpcMethod(name,this);
#endif

}

Method::~Method() {
#ifndef FAKIIR_GENESIS
  // delete transfers
  delete xmlrpc;
#endif
}

bool Method::exec() {

	if(!gateway) {
		fprintf(stderr,"ERROR: no exec gateway for method %s\n",
				name.c_str());
		return false;
	}

	(*gateway)(parent->real, &args);

	return true;

}


/////

Value::Value() {
	type = 0;
	value = NULL;
}

Value::~Value() {
	if(value) free(value);
}

bool Value::parse(char *desc) {
	// analize the description and form the value
	bool res = true;
	char *tok;
	char *sep = " \t\n";
	char *str = strdup(desc);

	tok = strtok(str,sep);
	while(tok) {

		if(strcmp( tok, "int") ==0
		|| strcmp( tok, "bool") ==0) {
			type = INT_VALUE;
			value = calloc(1,sizeof(int));
			break;
		} else if(strcmp( tok, "float") ==0) {
			type = FLOAT_VALUE;
			value = calloc(1,sizeof(float));
			break;
		} else if(strcmp( tok, "char") ==0) {
			type = STRING_VALUE;
			value = calloc(1,sizeof(char*));
			break;
		}

		tok = strtok(NULL,sep);
	
	}

	if(!type) {
		fprintf(stderr,"ERROR: malformed value %s\n",desc);
		res = false;
	}

	if(!tok) {
		fprintf(stderr,"ERROR: can't find value name in %s\n",desc);
		res = false;
	}

	tok = strtok(NULL,sep);

	if(!tok) {
		fprintf(stderr,"ERROR: no name for %s\n",desc);
		name = "unnamed";
	} else {
		if(type==STRING_VALUE && tok[0]=='*') tok++;
		name = tok; // implicit cast from char* to c++ string
	}

	free(str);

	fprintf(stderr,"\t\tVALUE \"%s\" of type %s\n", name.c_str(),
		(type==INT_VALUE)?"INT":
		(type==FLOAT_VALUE)?"FLOAT":
		(type==STRING_VALUE)?"STRING":"UNKNOWN");

	return res;
}

bool Value::set(char *arg) {
  bool res = true;
  
  if(type == INT_VALUE) {
    
    if( sscanf(arg, "%i", (int*)value) ==0)
      res = false;
    
  } else if(type == FLOAT_VALUE) {
    
    if( sscanf(arg, "%f", (float*)value) == 0)
      res = false;
    
  } else if(type == STRING_VALUE) {
    
    value = (void*)arg;
    
  } else {
    fprintf(stderr,"ERROR: value %s->set(%s) : no arg type\n",
	    name.c_str(), arg);
    res = false;
  }
  
  return res;
}

/////


Argument::Argument() 
  :Value() {
	method = NULL;
}

Argument::~Argument() { }


/////


Property::Property()
  :Value() {
	// TODO: parse the name and recognize type
	parent = NULL;
}

Property::~Property() { }


